@@ -29,6 +29,7 @@
 #define GRIDDIM ALBERTA_DIM
 #endif
 
+#include <dune/common/exceptions.hh>
 #include <dune/grid/onedgrid.hh>
 #include <dune/grid/common/gridfactory.hh>
 #include <dune/grid/test/gridcheck.hh>
@@ -47,7 +48,9 @@ struct EnableLevelIntersectionIteratorCheck< Dune::AlbertaGrid< dim, dimworld >
 #endif
 
 template <typename GridType>
-void testReadingAndWritingGrid( const std::string& path, const std::string& gridName, const std::string& gridManagerName, int refinements)
+void testReadingAndWritingGrid( const std::string& path, const std::string& gridName,
+                                const std::string& gridManagerName, int refinements,
+                                bool expectsBoundarySegments = false)
 {
   // Read the grid
   std::cout<<"Using "<<gridManagerName<<std::endl;
@@ -70,6 +73,9 @@ void testReadingAndWritingGrid( const std::string& path, const std::string& grid
           tempIDs[intersection.boundarySegmentIndex()]=boundaryIDs[gridFactory.insertionIndex(intersection)];
     boundaryIDs=std::move(tempIDs);
   }
+  else
+    if (expectsBoundarySegments)
+        DUNE_THROW(Dune::IOError, "Expected boundary segment markers found none!");
 
   // Load balancing and refinement
   grid->loadBalance();
@@ -82,25 +88,35 @@ void testReadingAndWritingGrid( const std::string& path, const std::string& grid
   // Write MSH
   Dune::GmshWriter<typename GridType::LeafGridView> writer( leafGridView );
   writer.setPrecision(10);
-  const std::string outputName("./"+gridName+"-"+gridManagerName+"-gmshtest-write.msh");
+  const std::string outputName(gridName+"-"+gridManagerName+"-gmshtest-write.msh");
   writer.write(outputName);
   if(!elementsIDs.empty())
   {
-    const std::string outputNameEntity("./"+gridName+"-"+gridManagerName+"-gmshtest-write-entity.msh");
+    const std::string outputNameEntity(gridName+"-"+gridManagerName+"-gmshtest-write-entity.msh");
     writer.write(outputNameEntity,elementsIDs);
   }
   if((!boundaryIDs.empty())&&(!elementsIDs.empty()))
   {
-    const std::string outputNameBoundary("./"+gridName+"-"+gridManagerName+"-gmshtest-write-boundary.msh");
+    const std::string outputNameBoundary(gridName+"-"+gridManagerName+"-gmshtest-write-boundary.msh");
     writer.write(outputNameBoundary,elementsIDs,boundaryIDs);
   }
 
   // Write VTK
   std::ostringstream vtkName;
-  vtkName << "./" << gridName << "-gmshtest-" << refinements;
+  vtkName << gridName << "-gmshtest-" << refinements;
   VTKWriter<typename GridType::LeafGridView> vtkWriter( leafGridView );
   vtkWriter.write( vtkName.str() );
   std::cout<<std::endl;
+
+  // Test whether grid can be read without giving the gridfactory explicitly
+  std::unique_ptr<GridType> gridUnique = GmshReader<GridType>::read(inputName);
+
+  // Remove the following tests once that GmshReader::read returns a std::unique_ptr
+  DUNE_NO_DEPRECATED_BEGIN
+  GridType* gridPtr = GmshReader<GridType>::read(inputName);
+  delete gridPtr;
+  DUNE_NO_DEPRECATED_END
+  std::shared_ptr<GridType> gridShared = GmshReader<GridType>::read(inputName);
 }
 
 
@@ -132,7 +148,7 @@ try
 #endif
 
 #if GMSH_ONEDGRID
-  testReadingAndWritingGrid<OneDGrid>( path, "oned-testgrid", "OneDGrid", refinements );
+  testReadingAndWritingGrid<OneDGrid>( path, "oned-testgrid", "OneDGrid", refinements, true);
 #endif
 
   return 0;
