@@ -5,12 +5,35 @@
 
 #include <memory>
 #include <vector>
+#include <fstream>
+#include <string>
 #include <unistd.h>
 
 // dune headers
 #include <dune/grid/yaspgrid.hh>
 #include <dune/grid/io/file/vtk/vtksequencewriter.hh>
 
+// check that the files have equal amount of lines
+void checkFileEqualNumLines(const std::string& name1, const std::string& name2)
+{
+  std::ifstream pvdFile1(name1);
+  std::ifstream pvdFile2(name2);
+
+  if (pvdFile1.fail())
+    DUNE_THROW(Dune::Exception, "File " << name1 << " could not be opened!");
+
+  if (pvdFile2.fail())
+    DUNE_THROW(Dune::Exception, "File " << name2 << " could not be opened!");
+
+  std::string line;
+  unsigned int lines1 = 0, lines2 = 0;
+  while (std::getline(pvdFile1, line)) ++lines1;
+  while (std::getline(pvdFile2, line)) ++lines2;
+
+  if (lines1 != lines2)
+    DUNE_THROW(Dune::Exception, "Not the same number of lines (comparing " << name1 << " and " << name2 << ")");
+}
+
 std::string VTKDataMode(Dune::VTK::DataMode dm)
 {
   switch(dm)
@@ -63,7 +86,8 @@ public:
 };
 
 template< class GridView >
-void doWrite( const GridView &gridView, Dune::VTK::DataMode dm )
+std::string doWrite( const GridView &gridView, Dune::VTK::DataMode dm,
+                     bool testRestart = false)
 {
   enum { dim = GridView :: dimension };
 
@@ -74,6 +98,9 @@ void doWrite( const GridView &gridView, Dune::VTK::DataMode dm )
   std::stringstream name;
   name << "vtktest-" << dim << "D-" << VTKDataMode(dm);
 
+  if (testRestart)
+    name << "-restart";
+
   auto vtkWriter = std::make_shared<Dune::VTKWriter<GridView> >(gridView, dm);
   Dune :: VTKSequenceWriter< GridView > vtk( vtkWriter, name.str(), ".", "" );
 
@@ -82,22 +109,47 @@ void doWrite( const GridView &gridView, Dune::VTK::DataMode dm )
   auto vectordata = std::make_shared<VTKVectorFunction<GridView> >();
   vtk.addVertexData(vectordata);
   double time = 0;
-  while (time<1) {
+  double tEnd = testRestart ? 0.5 : 1.0;
+  while (time < tEnd) {
     vectordata->setTime(time);
     vtk.write(time);
     time += 0.1;
   }
+
+  if (testRestart)
+  {
+    const auto& timeSteps = vtk.getTimeSteps();
+    auto vtkWriter2 = std::make_shared<Dune::VTKWriter<GridView> >(gridView, dm);
+    Dune :: VTKSequenceWriter< GridView > vtk2( vtkWriter2, name.str(), ".", "" );
+    vtk2.setTimeSteps(timeSteps);
+
+    vtk2.addVertexData(vertexdata,"vertexData");
+    vtk2.addCellData(celldata,"cellData");
+    vtk2.addVertexData(vectordata);
+
+    while (time < 1) {
+      vectordata->setTime(time);
+      vtk2.write(time);
+      time += 0.1;
+    }
+  }
+
+  return name.str();
 }
 
 template<int dim>
 void vtkCheck(const std::array<int,dim>& n,
-              const Dune::FieldVector<double,dim>& h)
+              const Dune::FieldVector<double,dim>& h,
+              bool testRestart = false)
 {
   std::cout << std::endl << "vtkSequenceCheck dim=" << dim << std::endl << std::endl;
   Dune::YaspGrid<dim> g(h, n);
   g.globalRefine(1);
 
-  doWrite( g.leafGridView(), Dune::VTK::conforming );
+  const auto fileName = doWrite( g.leafGridView(), Dune::VTK::conforming ) + ".pvd";
+  const auto fileNameRestart = doWrite( g.leafGridView(), Dune::VTK::conforming, testRestart ) + ".pvd";
+  checkFileEqualNumLines(fileName, fileNameRestart);
+
   doWrite( g.leafGridView(), Dune::VTK::nonconforming );
   doWrite( g.levelGridView( 0 ), Dune::VTK::conforming );
   doWrite( g.levelGridView( 0 ), Dune::VTK::nonconforming );
@@ -128,7 +180,7 @@ int main(int argc, char **argv)
     {
       std::array<int,3> n = { { 5, 5, 5 } };
       Dune::FieldVector<double,3> h = { 1.0, 2.0, 3.0 };
-      vtkCheck<3>(n,h);
+      vtkCheck<3>(n,h, /*testRestart=*/true);
     }
 
   } catch (Dune::Exception &e) {
