@@ -12,7 +12,7 @@
 #include <string>
 #include <vector>
 
-#include <dune/common/array.hh>
+#include <dune/common/std/make_array.hh>
 #include <dune/common/fvector.hh>
 #include <dune/common/parallel/mpihelper.hh>
 
@@ -103,13 +103,37 @@ int doWrite( Dune::VTKChecker& vtkChecker, const std::string& gridViewName, cons
   const typename GridView :: IndexSet &is = gridView.indexSet();
   std::vector<int> vertexdata(is.size(dim),dim);
   std::vector<int> celldata(is.size(0),0);
+  std::vector<double> celldoubledata(is.size(0), 1.00000000000346e-12);
 
+  // Write algebraic data
   vtk.addVertexData(vertexdata,"vertexData");
   vtk.addCellData(celldata,"cellData");
+  vtk.addCellData(celldoubledata, "cellData_double", 1, Dune::VTK::Precision::float64);
 
+  // Write functions given as VTKFunction objects
   vtk.addVertexData(std::make_shared< VTKVectorFunction<GridView> >("vertex"));
   vtk.addCellData(std::make_shared< VTKVectorFunction<GridView> >("cell"));
 
+  // Write globally defined C++ functions
+  auto f1 = [](const Dune :: FieldVector<double,dim>& x)
+  {
+    return std::sin(x.two_norm());
+  };
+
+  vtk.addCellData(f1,
+                  Dune :: VTK :: FieldInfo("scalar-valued lambda",
+                                           Dune :: VTK :: FieldInfo :: Type :: scalar, 1));
+
+  auto f2 = [](const Dune :: FieldVector<double,dim>& x)
+  {
+    return x;
+  };
+
+  vtk.addVertexData(f2,
+                    Dune :: VTK :: FieldInfo("vector-valued lambda",
+                                             Dune :: VTK :: FieldInfo :: Type :: vector, 2,
+                                             Dune :: VTK :: Precision::float64));
+
   int result = 0;
   std::string name;
   std::ostringstream prefix;
@@ -172,7 +196,7 @@ int main(int argc, char **argv)
                 << std::endl;
 
     int result = 0; // pass by default
-    using Dune::make_array;
+    using Dune::Std::make_array;
 
     Dune::VTKChecker vtkChecker;
 
