@@ -9,6 +9,10 @@
 
 #include <dune/grid/uggrid.hh>
 
+#if ModelP and DUNE_UGGRID_HAVE_PPIFCONTEXT
+#  include <dune/uggrid/parallel/ppif/ppifcontext.hh>
+#endif
+
 /** \todo Remove the following two includes once getAllSubfaces... is gone */
 #include <list>
 #include <iterator>
@@ -27,13 +31,11 @@ namespace Dune {
 template<> int UGGrid<2>::numOfUGGrids = 0;
 template<> int UGGrid<3>::numOfUGGrids = 0;
 
-template<> unsigned int UGGrid<2>::heapSize_ = 500;
-template<> unsigned int UGGrid<3>::heapSize_ = 500;
-
 
 template <int dim>
-UGGrid < dim >::UGGrid()
+UGGrid < dim >::UGGrid(UGCollectiveCommunication comm)
   : multigrid_(nullptr),
+    ccobj_(comm),
     leafIndexSet_(*this),
     idSet_(*this),
     refinementType_(LOCAL),
@@ -185,7 +187,7 @@ template < int dim >
 bool UGGrid < dim >::mark(int refCount,
                           const typename Traits::template Codim<0>::Entity& e )
 {
-  typename UG_NS<dim>::Element* target = this->getRealImplementation(e).target_;
+  typename UG_NS<dim>::Element* target = e.impl().target_;
 
   // No refinement requested
   if (refCount==0) {
@@ -227,7 +229,7 @@ bool UGGrid < dim >::mark(const typename Traits::template Codim<0>::Entity& e,
                           typename UG_NS<dim>::RefinementRule rule,
                           int side)
 {
-  typename UG_NS<dim>::Element* target = this->getRealImplementation(e).target_;
+  typename UG_NS<dim>::Element* target = e.impl().target_;
 
   if (!UG_NS<dim>::isLeaf(target))
     return false;
@@ -241,7 +243,7 @@ bool UGGrid < dim >::mark(const typename Traits::template Codim<0>::Entity& e,
 template <int dim>
 int UGGrid<dim>::getMark(const typename Traits::template Codim<0>::Entity& e) const
 {
-  typename UG_NS<dim>::Element* target = this->getRealImplementation(e).target_;
+  typename UG_NS<dim>::Element* target = e.impl().target_;
 
   // Return -1 if element is marked for coarsening
   if (UG_NS<dim>::ReadCW(target,UG_NS<dim>::COARSEN_CE))
@@ -324,7 +326,7 @@ void UGGrid <dim>::postAdapt()
 {
   for (int i=0; i<=maxLevel(); i++)
     for (const auto& element : elements(this->levelGridView(i)))
-      UG_NS<dim>::WriteCW(this->getRealImplementation(element).target_, UG_NS<dim>::NEWEL_CE, 0);
+      UG_NS<dim>::WriteCW(element.impl().target_, UG_NS<dim>::NEWEL_CE, 0);
 
   // reset marker flags
   someElementHasBeenMarkedForRefinement_ = false;
@@ -373,7 +375,7 @@ void UGGrid<dim>::getChildrenOfSubface(const typename Traits::template Codim<0>:
   if (!e.isLeaf()   // Get_Sons_of_ElementSide returns GM_FATAL when called for a leaf !?!
       && e.level() < maxl) {
 
-    typename UG_NS<dim>::Element* theElement = this->getRealImplementation(e).target_;
+    typename UG_NS<dim>::Element* theElement = e.impl().target_;
     list.emplace_back(theElement, elementSide);
 
   }
@@ -465,7 +467,7 @@ bool UGGrid < dim >::loadBalance(const std::vector<Rank>& targetProcessors, unsi
     return true;
 
 #ifdef ModelP  // The Partition field only exists if ModelP is set
-  if (targetProcessors.size() != this->leafGridView().size(0))
+  if (int(targetProcessors.size()) != this->leafGridView().size(0))
     DUNE_THROW(Exception, "targetProcessors argument does not have the correct size");
 
   // Get unique consecutive index across different element types
@@ -480,14 +482,14 @@ bool UGGrid < dim >::loadBalance(const std::vector<Rank>& targetProcessors, unsi
 
       if (element.isLeaf()) {
 
-        auto targetRank = targetProcessors[elementMapper.index(element)];
+        int targetRank = targetProcessors[elementMapper.index(element)];
 
         // sanity check
         if (targetRank >= comm().size())
           DUNE_THROW(GridError, "Requesting target processor " << targetRank <<
                      ", but only " << comm().size() << " processors are available.");
 
-        UG_NS<dim>::Partition(this->getRealImplementation(element).target_) = targetRank;
+        UG_NS<dim>::Partition(element.impl().target_) = targetRank;
       } else {
 
         std::map<Rank,unsigned int> rank;
@@ -497,7 +499,7 @@ bool UGGrid < dim >::loadBalance(const std::vector<Rank>& targetProcessors, unsi
         // Loop over all children and collect the ranks they are assigned to
         for (const auto& child : descendantElements(element, element.level() + 1)) {
 
-          auto childRank = UG_NS<dim>::Partition(this->getRealImplementation(child).target_);
+          auto childRank = UG_NS<dim>::Partition(child.impl().target_);
 
           if (rank.find(childRank) == rank.end())
             rank[childRank] = 1;
@@ -512,7 +514,7 @@ bool UGGrid < dim >::loadBalance(const std::vector<Rank>& targetProcessors, unsi
         }
 
         // Assign rank that occurred most often
-        UG_NS<dim>::Partition(this->getRealImplementation(element).target_) = mostFrequentRank;
+        UG_NS<dim>::Partition(element.impl().target_) = mostFrequentRank;
 
       }
     }
@@ -536,7 +538,7 @@ template < int dim >
 void UGGrid < dim >::setPosition(const typename Traits::template Codim<dim>::Entity& e,
                                  const FieldVector<double, dim>& pos)
 {
-  typename UG_NS<dim>::Node* target = this->getRealImplementation(e).target_;
+  typename UG_NS<dim>::Node* target = e.impl().target_;
 
   for (int i=0; i<dim; i++)
     target->myvertex->iv.x[i] = pos[i];
@@ -576,28 +578,36 @@ void UGGrid<dim>::loadState(const std::string& filename)
 
   if (dim==2) {
     std::string formatName = "DuneFormat2d";
-    multigrid_ = (typename UG_NS<dim>::MultiGrid*) UG::D2::LoadMultiGrid(name_.c_str(),
-                                                                         filename.c_str(),
-                                                                         type,
-                                                                         problemName.c_str(),
-                                                                         formatName.c_str(),
-                                                                         heapSize_,
-                                                                         true, //force,
-                                                                         true, //optimizedIO,
-                                                                         false //autosave
-                                                                         );
+    multigrid_ = (typename UG_NS<dim>::MultiGrid*) UG::D2::LoadMultiGrid(
+      name_.c_str(),
+      filename.c_str(),
+      type,
+      problemName.c_str(),
+      formatName.c_str(),
+      0,    // dummy heap size
+      true, //force,
+      true, //optimizedIO,
+      false //autosave
+#if ModelP and DUNE_UGGRID_HAVE_PPIFCONTEXT
+      , std::make_shared<PPIF::PPIFContext>(comm())
+#endif
+      );
   } else {
     std::string formatName = "DuneFormat3d";
-    multigrid_ = (typename UG_NS<dim>::MultiGrid*) UG::D3::LoadMultiGrid(name_.c_str(),
-                                                                         filename.c_str(),
-                                                                         type,
-                                                                         problemName.c_str(),
-                                                                         formatName.c_str(),
-                                                                         heapSize_,
-                                                                         true, //force,
-                                                                         true, //optimizedIO,
-                                                                         false //autosave
-                                                                         );
+    multigrid_ = (typename UG_NS<dim>::MultiGrid*) UG::D3::LoadMultiGrid(
+      name_.c_str(),
+      filename.c_str(),
+      type,
+      problemName.c_str(),
+      formatName.c_str(),
+      0,    // dummy heap size
+      true, //force,
+      true, //optimizedIO,
+      false //autosave
+#if ModelP and DUNE_UGGRID_HAVE_PPIFCONTEXT
+      , std::make_shared<PPIF::PPIFContext>(comm())
+#endif
+      );
   }
 
   if (multigrid_==nullptr)
