@@ -5,13 +5,15 @@
 
 #include <memory>
 
-#include <dune/common/std/memory.hh>
 #include <dune/common/stdstreams.hh>
-#include <dune/common/parallel/mpihelper.hh>
 
 #include <dune/grid/uggrid/uggridfactory.hh>
 #include "boundaryextractor.hh"
 
+#if ModelP and DUNE_UGGRID_HAVE_PPIFCONTEXT
+#  include <dune/uggrid/parallel/ppif/ppifcontext.hh>
+#endif
+
 namespace Dune {
 
 /* The following three methods are the ones that UG calls to know about the geometry
@@ -213,7 +215,7 @@ insertBoundarySegment(const std::vector<unsigned int>& vertices,
 }
 
 template <int dimworld>
-UGGrid<dimworld>* GridFactory<UGGrid<dimworld> >::
+ToUniquePtr<UGGrid<dimworld>> GridFactory<UGGrid<dimworld> >::
 createGrid()
 {
   // Prevent a crash when this method is called twice in a row
@@ -236,23 +238,25 @@ createGrid()
   // on all processes.
   // ///////////////////////////////////////////////////////////////////////////////
 
+  const auto& comm = grid_->comm();
+
   // Broadcast the vertex positions
   int numVertices = vertexPositions_.size();
-  MPIHelper::getCollectiveCommunication().broadcast(&numVertices, 1, 0);
+  comm.broadcast(&numVertices, 1, 0);
   vertexPositions_.resize(numVertices);
-  MPIHelper::getCollectiveCommunication().broadcast(&vertexPositions_[0], vertexPositions_.size(), 0);
+  comm.broadcast(&vertexPositions_[0], vertexPositions_.size(), 0);
 
   // Broadcast the element corners
   int numElementVertices = elementVertices_.size();
-  MPIHelper::getCollectiveCommunication().broadcast(&numElementVertices, 1, 0);
+  comm.broadcast(&numElementVertices, 1, 0);
   elementVertices_.resize(numElementVertices);
-  MPIHelper::getCollectiveCommunication().broadcast(&elementVertices_[0], elementVertices_.size(), 0);
+  comm.broadcast(&elementVertices_[0], elementVertices_.size(), 0);
 
   // Broadcast the geometry types
   int numElementTypes = elementTypes_.size();
-  MPIHelper::getCollectiveCommunication().broadcast(&numElementTypes, 1, 0);
+  comm.broadcast(&numElementTypes, 1, 0);
   elementTypes_.resize(numElementTypes);
-  MPIHelper::getCollectiveCommunication().broadcast(&elementTypes_[0], elementTypes_.size(), 0);
+  comm.broadcast(&elementTypes_[0], elementTypes_.size(), 0);
 
   // ///////////////////////////////////////////
   //   Extract grid boundary segments
@@ -454,20 +458,24 @@ createGrid()
     DUNE_THROW(GridError, "Calling UG::" << dimworld << "d::ConfigureCommand failed!");
 
   //new @PROBLEM $b @PROBLEM $f @FORMAT $h @HEAP;
-  char* newArgs[4];
-  for (int i=0; i<4; i++)
+  char* newArgs[3];
+  for (int i=0; i<3; i++)
     newArgs[i] = (char*)::malloc(50*sizeof(char));
 
   sprintf(newArgs[0], "new %s", grid_->name_.c_str());
 
   sprintf(newArgs[1], "b %s_Problem", grid_->name_.c_str());
   sprintf(newArgs[2], "f DuneFormat%dd", dimworld);
-  sprintf(newArgs[3], "h %dM", grid_->heapSize_);
 
-  if (UG_NS<dimworld>::NewCommand(4, newArgs))
+  if (UG_NS<dimworld>::NewCommand(
+        3, newArgs
+#if ModelP and DUNE_UGGRID_HAVE_PPIFCONTEXT
+        , std::make_shared<PPIF::PPIFContext>(grid_->comm())
+#endif
+        ))
     DUNE_THROW(GridError, "UGGrid<" << dimworld << ">::makeNewMultigrid failed!");
 
-  for (int i=0; i<4; i++)
+  for (int i=0; i<3; i++)
     free(newArgs[i]);
 
   // Get a direct pointer to the newly created multigrid
@@ -479,7 +487,12 @@ createGrid()
   // If we are in a parallel setting and we are _not_ the master
   // process we can stop here.
   // ///////////////////////////////////////////////////////////////
-  if (PPIF::me!=0) {
+#if ModelP and DUNE_UGGRID_HAVE_PPIFCONTEXT
+  const bool isMaster = grid_->multigrid_->ppifContext().isMaster();
+#else
+  const bool isMaster = (PPIF::me == PPIF::master);
+#endif
+  if (not isMaster) {
     // Complete the UG-internal grid data structure even if we are
     // not the master process. (CreateAlgebra communicates via MPI
     // so we would be out of sync if we don't do this here...)
@@ -493,7 +506,7 @@ createGrid()
     grid_->levelIndexSets_[0] = std::make_shared<UGGridLevelIndexSet<const UGGrid<dimworld> > >();
 
     /* here all temp memory since CreateMultiGrid is released */
-    Release(grid_->multigrid_->theHeap, UG::FROM_TOP, grid_->multigrid_->MarkKey);
+    ReleaseTmpMem(grid_->multigrid_->theHeap, grid_->multigrid_->MarkKey);
     grid_->multigrid_->MarkKey = 0;
 
     // ///////////////////////////////////////////////////
@@ -548,7 +561,7 @@ createGrid()
     DUNE_THROW(IOError, "Call of 'UG::D" << dimworld << "::CreateAlgebra' failed!");
 
   /* here all temp memory since CreateMultiGrid is released */
-  Release(grid_->multigrid_->theHeap, UG::FROM_TOP, grid_->multigrid_->MarkKey);
+  ReleaseTmpMem(grid_->multigrid_->theHeap, grid_->multigrid_->MarkKey);
   grid_->multigrid_->MarkKey = 0;
 
   // Set the local indices
@@ -556,7 +569,7 @@ createGrid()
 
   // Clear refinement flags
   for (const auto& element : elements(grid_->levelGridView(0)))
-    UG_NS<dimworld>::WriteCW(grid_->getRealImplementation(element).target_, UG_NS<dimworld>::NEWEL_CE, 0);
+    UG_NS<dimworld>::WriteCW(element.impl().target_, UG_NS<dimworld>::NEWEL_CE, 0);
 
 
   // ///////////////////////////////////////////////////
